/***************************************************************************
 *                                                                         *
 *   LB - The Little Budget Utility for GnuCash                            *
 *                                                                         *
 *   Copyright (C) 2004 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


/* main.cpp
   Gabriel M. Beddingfield
   02OCT2004

   For merging a gnucash report
   with a published budget.
*/

#include <iostream>
#include <iomanip>
#include <string>
#include "bud.h"
#include "budget.h"
#include "report.h"

using namespace std;

#ifndef VERSION
#define VERSION "0.1.0"
#endif

void usage(void)
{
  cout << endl;
  cout << "  lb [options] -b budget.txt -r report.html" << endl;
  cout << endl;
  cout << "  Options:" << endl;
  cout << endl;
  cout << "  -b budget.txt -- Text file with the budget figures." << endl;
  cout << endl;
  cout << "  -r report.html -- Cash Flow report from Gnucash." << endl;
  cout << endl;
  cout << "  -s N -- Scale factor for budget.  All figures will be" << endl;
  cout << "          multiplied by N." << endl;
  cout << endl;
  cout << "  -q -- quiet operation.  No display of title or version." << endl;
  cout << endl;
}

int report_accounts( accounts& accts, const float k ) {

#define ONELINE(_a,_b,_c,_d)  cout << setw(37) << (_a) << "  " << \
        setw(9) << setprecision(2) << setiosflags(ios::right|ios::fixed) << \
        (_b) << \
        " " << setw(9) << (_c) << \
        " " << setw(9) << (_d) << endl \

  enum { income=0, assets=1, expenses=2 };
  enum { budget=0, actual=1 };

  // Note:  sums contains the values in the following array:
  // sums[][] =  [ income_budgeted  income_actual
  //               assets_budgeted  assets_actual
  //               expenses_budget  expenses_actual ]

  float sums[3][2] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
  string str[3] = { "Income", "Assets", "Expenses" };
  string tot = "Total ";
  string col = ":";

  //Headers
  cout << endl;
  ONELINE("Account","Planned","Actual","Diff");
  ONELINE("=====================================",
	  "=========","=========","=========");

  // put income at the top
  accounts::iterator cur;
  bool test;

  for(int j=0; j<3; j++) {
    cur = accts.begin();
    for(int n=0; n<accts.size(); n++) {
      if(j<expenses) {
	test = (0==(*cur).first.compare(0,str[j].length(),str[j]));
      }else{
	test = ( (*cur).first.compare(0,str[income].length(),str[income]) &&
		 (*cur).first.compare(0,str[assets].length(),str[assets]) );
      }
      if(test) {
	ONELINE((*cur).first,k*(*cur).second.planned(),(*cur).second.actual(),
		(*cur).second.diff(k));
	sums[j][budget] += k*(*cur).second.planned();
	sums[j][actual] += (*cur).second.actual();
	}
      cur++;
    }
    ONELINE("","=========","=========","=========");
    ONELINE(tot+str[j]+col,sums[j][budget],sums[j][actual],
	    sums[j][actual]-sums[j][budget]);
    cout << endl;
  }

  cout << endl;
  float bbl=sums[income][budget]-sums[assets][budget]-sums[expenses][budget];
  float abl=sums[income][actual]-sums[assets][actual]-sums[expenses][actual];
  ONELINE("BOTTOM LINE:",bbl,abl,abl-bbl);

  cout << endl;

#undef ONELINE

  return 0;
}

int main( int argc, char* argv[] )
{
  int b=0, r=0, s=0, q=0;
  float k=1.0;

  // parse the command line
  for(int i=1;i<argc;i++) {
    if(0==strcmp(argv[i],"-b")) { b=i+1; continue; }
    if(0==strcmp(argv[i],"-r")) { r=i+1; continue; }
    if(0==strcmp(argv[i],"-s")) { s=i+1; continue; }
    if(0==strcmp(argv[i],"-q")) { q=1;   continue; }
  }

  if(!q) {
  cout << "LB - The Little Budget Tool " << VERSION << 
          " (C)2004 by G. M. Beddingfield, Romans 11:36" << endl;
  }

  // Check for bad inputs.
  if((b>=argc)||(r>=argc)||(s>=argc)||!(b||r)) {
    usage();
    return 0;
  }

  accounts accts;
  string str;

  if(s) {
    k = atof(argv[s]);
  }

  if(b) {
    ReadBudget bfile(argv[b]);
  
    while(bfile.more()) {
      accts[bfile.acct()].planned(bfile.val());
    }
  } // if(b)

  if(r) {
    ReadReport rfile(argv[r]);

    cout << "Beginning date: " << rfile.beginning() << "." << endl;
    cout << "Ending date:    " << rfile.ending() << "." << endl;

    while(rfile.more()) {
      accts[rfile.acct()].actual(rfile.val());
    }
  } // if(r)


  return report_accounts(accts,k);

}
