/***************************************************************************
 *                                                                         *
 *   LB - The Little Budget Utility for GnuCash                            *
 *                                                                         *
 *   Copyright (C) 2004 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* report.cpp
   Gabriel M. Beddingfield
   03OCT2004

   Definition of ReadReport object

   TO-DO:
   HTML tags may be upper or lower-case.  However,
   this code only searches for upper-case tags.
   Perhaps this should be more robust.
*/

#include "report.h"
#include <cstring>
#include <iostream>

using namespace std;

#define BUFFER 256

// Constructor.
// Opens file and reads in dates.
ReadReport::ReadReport(const char * str) : ifstream(str), val_(0.0) {
  if(!good()||eof()) return;

  char buf[BUFFER]="";
  char srch[BUFFER]="<H3>Cash Flow - ";
  char* ptr;
  int N = strlen(srch);

  // Seek the line with the dates.
  while((0!=strncmp(buf,srch,N))&&(!eof())) {
    getline(buf,BUFFER);
  }

  ptr = strtok(buf+N," ");
  beg_ = ptr;
  ptr = strtok(NULL," ");
  ptr = strtok(NULL," ");
  end_ = ptr;

  return;
}
  
bool ReadReport::more(void) {
  static char buf[BUFFER];
  static char srch[BUFFER];
  static bool head=false;
  static bool credits=false;
  static bool debits=true;  // will switch to false when credits toggles

  int N;
  int i;
  
  val_=0;

  if((!good()||eof())||(head&&credits&&debits)) {
    close();
    return false;
  }
 
  if(!head) {
    buf[0]='\0';
    strcpy(srch,"<HR");
    N = strlen(srch);

    // First step is to jump to the first <HR>
    while((0!=strncmp(buf,srch,N))&&(!eof())) {
      getline(buf,BUFFER);
    }

    // Burn through one <TR> section
    strcpy(srch,"<TR");
    N = strlen(srch);
    for(i=0;i<2;i++) {
      buf[0]='\0';
      while((0!=strncmp(buf,srch,N))&&(!eof())) {
        getline(buf,BUFFER);
      }
    }

    head=true;
  } // if(!head)

  if(!credits) {

    // Process one account
    credits = oneacct();

    // If there are no more credits, then
    // it's time to switch to debits.
    if(!credits) {
      // Negate value of any expense accounts.
      strcpy(srch,"Expenses");
      N=strlen(srch);
      if(0==strncmp(name_,srch,N)) val_ = -val_;
      // Or "Asset transfers"
      strcpy(srch,"Assets");
      N=strlen(srch);
      if(0==strncmp(name_,srch,N)) val_ = -val_;
    }else{
      debits = false;
      // scan to <HR> and then burn through one <TR>
      buf[0]='\0';
      strcpy(srch,"<HR");
      N = strlen(srch);
      while((0!=strncmp(buf,srch,N))&&(!eof())) {
	getline(buf,BUFFER);
      }
      strcpy(srch,"<TR");
      N = strlen(srch);
      for(i=0;i<2;i++) {
	buf[0]='\0';
	while((0!=strncmp(buf,srch,N))&&(!eof())) {
	  getline(buf,BUFFER);
	}
      }
    }  // if(!credits)
  } // if(!credits)

  if(!debits) {

    // process one account
    debits = oneacct();
    if(debits) {
      close();
      return false;
    }
  } // if(!debits)

  return true;
}

bool ReadReport::oneacct(void) {

  // assume that we've already been placed at a <TR>
  // The upcoming section either has data, or it is
  // a transition between section.  We are either
  // looking for "<A HREF=", "$", or "Money".  The
  // HREF points us to an account name.  The $ points
  // us to the value, and "Money" signals the end.
  static char buf[BUFFER];
  static const char href[]="<A HREF=";
  static const int hN=strlen(href);
  static const char dollar[]="$";
  static const int dN=strlen(dollar);
  static const char money[]="Money";
  static const int mN=strlen(money);
  char* ptr;

  buf[0]='\0';

  int i=0;

  while(strncmp(buf,href,hN)&&strncmp(buf,dollar,dN)
	&&strncmp(buf,money,mN)&&!eof()&&(i<2)) {
    getline(buf,BUFFER);

    if(eof()) {
      close();
      return true;
    }

    //Case of href field
    if(0==strncmp(buf,href,hN)) {
      ptr=strtok(buf,">");
      name_=strtok(NULL,"<");
      i++;
    }

    //Case of $
    if(0==strncmp(buf,dollar,dN)) {
      // kill the comma
      ptr=buf+1;
      while((*ptr!=',')&&(*ptr!='\0')) ptr++;
      if((*ptr)==',') {
	do {
	  (*ptr) = *(ptr+1);
	  ptr++;
	} while((*ptr)!='\0');
      }
      val_ = atof(buf+1);
      i++;
    }

    //Case of "Money"
    if(0==strncmp(buf,money,mN)) {
      return true;
    }
  } // main loop

  if(eof()) {
    close();
    return true;
  }

  return false;
}
  
